#pragma once

#include <QtWidgets/QLayout>
#include <QtWidgets/QStyle>

namespace AxiomGui {

    class FlowLayoutSorter {
    public:
        virtual bool aAfterB(QLayoutItem *a, QLayoutItem *b) = 0;
    };

    class FlowLayout : public QLayout {
    public:
        explicit FlowLayout(QWidget *parent, int margin = -1, int hSpacing = -1, int vSpacing = -1,
                            FlowLayoutSorter *sorter = nullptr);

        explicit FlowLayout(int margin = -1, int hSpacing = -1, int vSpacing = -1, FlowLayoutSorter *sorter = nullptr);

        ~FlowLayout() override;

        void addItem(QLayoutItem *item) override;

        int horizontalSpacing() const;

        int verticalSpacing() const;

        Qt::Orientations expandingDirections() const override;

        bool hasHeightForWidth() const override;

        int heightForWidth(int) const override;

        int count() const override;

        QLayoutItem *itemAt(int index) const override;

        QSize minimumSize() const override;

        void setGeometry(const QRect &rect) override;

        QSize sizeHint() const override;

        QLayoutItem *takeAt(int index) override;

    private:
        int doLayout(const QRect &rect, bool testOnly) const;

        int smartSpacing(QStyle::PixelMetric pm) const;

        QList<QLayoutItem *> itemList;
        int m_hSpace;
        int m_vSpace;
        FlowLayoutSorter *sorter;
    };
}
